@@ -2,7 +2,7 @@
 
 # System
 from typing import Optional, Union, List, Dict, Callable, Tuple
-import pickle, os, time, json
+import pickle, os, time, json, inspect
 
 # Pip
 from selenium import webdriver
@@ -13,6 +13,7 @@ from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.common.by import By as by
 from selenium.common.exceptions import NoSuchElementException
 from selenium.webdriver.firefox.options import Options as FirefoxOptions
+from selenium.webdriver.common.action_chains import ActionChains
 
 from fake_useragent import UserAgent
 import tldextract
@@ -101,46 +102,51 @@ class Firefox:
                     with open(user_agent_path, 'w') as file:
                         file.write(user_agent)
 
-            profile.set_preference("general.useragent.override", user_agent)
+            profile.set_preference('general.useragent.override', user_agent)
         
         if language is not None:
             profile.set_preference('intl.accept_languages', language)
 
         if private:
-            profile.set_preference("browser.privatebrowsing.autostart", True)
+            profile.set_preference('browser.privatebrowsing.autostart', True)
         
         if disable_images:
             profile.set_preference('permissions.default.image', 2)
             profile.set_preference('dom.ipc.plugins.enabled.libflashplayer.so', False)
         
         if host is not None and port is not None:
-            profile.set_preference("network.proxy.type", 1)
-            profile.set_preference("network.proxy.http", host)
-            profile.set_preference("network.proxy.http_port", port)
-            profile.set_preference("network.proxy.ssl", host)
-            profile.set_preference("network.proxy.ssl_port", port)
-            profile.set_preference("network.proxy.ftp", host)
-            profile.set_preference("network.proxy.ftp_port", port)
-            profile.set_preference("network.proxy.socks", host)
-            profile.set_preference("network.proxy.socks_port", port)
-            profile.set_preference("network.proxy.socks_version", 5)
-            profile.set_preference("signon.autologin.proxy", True)
+            profile.set_preference('network.proxy.type', 1)
+            profile.set_preference('network.proxy.http', host)
+            profile.set_preference('network.proxy.http_port', port)
+            profile.set_preference('network.proxy.ssl', host)
+            profile.set_preference('network.proxy.ssl_port', port)
+            profile.set_preference('network.proxy.ftp', host)
+            profile.set_preference('network.proxy.ftp_port', port)
+            profile.set_preference('network.proxy.socks', host)
+            profile.set_preference('network.proxy.socks_port', port)
+            profile.set_preference('network.proxy.socks_version', 5)
+            profile.set_preference('signon.autologin.proxy', True)
         
-        profile.set_preference("marionatte", False)
-        profile.set_preference("dom.webdriver.enabled", False)
-        profile.set_preference("media.peerconnection.enabled", False)
+        profile.set_preference('marionatte', False)
+        profile.set_preference('dom.webdriver.enabled', False)
+        profile.set_preference('media.peerconnection.enabled', False)
         profile.set_preference('useAutomationExtension', False)
 
-        profile.set_preference("general.warnOnAboutConfig", False)
+        profile.set_preference('general.warnOnAboutConfig', False)
         profile.update_preferences()
         options = FirefoxOptions()
 
-        if headless:
-            options.add_argument("--headless")
-        
         if screen_size is not None:
-            options.add_argument("--width=" + str(screen_size[0]))
-            options.add_argument("--height=" + str(screen_size[1]))
+            options.add_argument('--width={}'.format(screen_size[0]))
+            options.add_argument('--height={}'.format(screen_size[1]))
+            options.add_argument('--window-size={},{}'.format(screen_size[0], screen_size[1]))
+            options.add_argument('window-size={}x{}'.format(screen_size[0], screen_size[1]))
+
+        if full_screen:
+            options.add_argument('--start-maximized')
+
+        if headless:
+            options.add_argument('--headless')
 
         self.driver = webdriver.Firefox(firefox_profile=profile, firefox_options=options)
 
@@ -291,13 +297,13 @@ class Firefox:
         attributes: Optional[Dict[str, str]] = None,
         id_: Optional[str] = None,
         class_: Optional[str] = None,
-        in_element: Optional = None,
+        in_element: Optional[WebElement] = None,
         timeout: Optional[int] = None,
         **kwargs
     ) -> Optional[WebElement]:
         return self.find(
             By.XPATH,
-            self.generate_xpath(type_=type_, attributes=attributes, id_=id_, class_=class_, for_sub_element=in_element, **kwargs),
+            self.generate_xpath(type_=type_, attributes=attributes, id_=id_, class_=class_, for_sub_element=in_element is not None, **kwargs),
             element=in_element,
             timeout=timeout
         )
@@ -338,13 +344,13 @@ class Firefox:
         attributes: Optional[Dict[str, str]] = None,
         id_: Optional[str] = None,
         class_: Optional[str] = None,
-        in_element: Optional = None,
+        in_element: Optional[WebElement] = None,
         timeout: Optional[int] = None,
         **kwargs
     ) -> List[WebElement]:
         return self.find_all(
             By.XPATH,
-            self.generate_xpath(type_=type_, attributes=attributes, id_=id_, class_=class_, for_sub_element=in_element, **kwargs),
+            self.generate_xpath(type_=type_, attributes=attributes, id_=id_, class_=class_, for_sub_element=in_element is not None, **kwargs),
             element=in_element,
             timeout=timeout
         )
@@ -380,7 +386,7 @@ class Firefox:
 
         pickle.dump(
             cookies or self.driver.get_cookies(),
-            open(self.__cookies_path(), "wb")
+            open(self.__cookies_path(), 'wb')
         )
 
     def load_cookies(self) -> None:
@@ -389,7 +395,7 @@ class Firefox:
 
             return
 
-        cookies = pickle.load(open(self.__cookies_path(), "rb"))
+        cookies = pickle.load(open(self.__cookies_path(), 'rb'))
         should_save = False
         cookies_to_save = []
 
@@ -430,7 +436,7 @@ class Firefox:
     
     def scroll_to(self, position: int) -> None:
         try:
-            self.driver.execute_script("window.scrollTo(0,"+str(position)+");")
+            self.driver.execute_script('window.scrollTo(0,'+str(position)+');')
         except:
             pass
 
@@ -444,8 +450,39 @@ class Firefox:
             _, element_y, _, _, _, _ = self.get_element_coordinates(element)
 
             self.scroll_to(element_y-header_h)
-        except:
-            pass
+        except Exception as e:
+            print('scroll_to_element', e)
+    
+    def move_to_element(self, element: Optional[WebElement]) -> bool:
+        if not element:
+            print('move_to_element: None element passed')
+
+            return False
+
+        try:
+            actions = ActionChains(self.driver)
+            actions.move_to_element(element).perform()
+
+            return True
+        except Exception as e:
+            print('move_to_element', e)
+
+            return False
+
+    def switch_to_frame(self, iframe: Optional[WebElement]) -> bool:
+        if not iframe:
+            print('switch_to_frame: None frame passed')
+
+            return False
+
+        try:
+            self.driver.switch_to.frame(iframe)
+
+            return True
+        except Exception as e:
+            print('switch_to_frame', e)
+
+            return False
 
     # returns x, y, w, h, max_x, max_y
     def get_element_coordinates(self, element) -> Tuple[int, int, int, int, int, int]:
@@ -460,11 +497,11 @@ class Firefox:
         return x, y, w, h, x+w, y+h
 
     def current_page_offset_y(self) -> float:
-        return self.driver.execute_script("return window.pageYOffset;")
+        return self.driver.execute_script('return window.pageYOffset;')
 
     def open_new_tab(self, url: str) -> None:
         if url is None:
-            url = ""
+            url = ''
 
         cmd = 'window.open("'+url+'","_blank");'
         self.driver.execute_script(cmd)
@@ -500,6 +537,14 @@ class Firefox:
         return ('.' if for_sub_element else '') + '//' + type_ + (('[' + xpath_query + ']') if len(xpath_query) > 0 else '')
 
 
+    # JS
+    def js_click(self, element: WebElement) -> bool:
+        return self.__js_execute_on_element('arguments[0].click();', element)
+
+    def js_scroll_into_view(self, element: WebElement) -> bool:
+        return self.__js_execute_on_element('arguments[0].scrollIntoView();', element)
+
+
     # LEGACY
     def scroll_to_bottom(self) -> None:
         MAX_TRIES = 25
@@ -580,5 +625,21 @@ class Firefox:
             formatted_url + '.pkl'
         )
 
+    def __js_execute_on_element(self, script: str, element: WebElement) -> bool:
+        caller_name = inspect.stack()[2][3]
+    
+        if not element:
+            print('{}: passed element is None'.format(caller_name))
+
+            return False
+
+        try:
+            self.driver.execute_script(script, element)
+
+            return True
+        except Exception as e:
+            print('{}: {}'.format(caller_name, e))
+
+            return False
 
 # ---------------------------------------------------------------------------------------------------------------------------------------- #
\ No newline at end of file
